import os
import random
rr = random.randrange

import pygame
from pygame.locals import *

from pgu import engine

from cnst import *

class Sprite:
    def __init__(self,img,shape):
        self.rect = pygame.Rect(shape)
        self.shape = pygame.Rect(shape)
        self.image = img

def elephant_new(g,pos):
    #s = Sprite(g.gfx['elephant'],pygame.Rect(0,0,96,130))
    #s = Sprite(g.gfx['elephant'],pygame.Rect(28,8,28,48))
    s = Sprite(g.gfx['elephant'],pygame.Rect(43-14,8,28,48))
    s.rect.centerx = pos[0]
    s.rect.bottom = pos[1]
    
    g.sprites.append(s)
    
    s.ball = ball_new(g,(s.rect.centerx,s.rect.bottom))
    s.loop = elephant_loop
    s.vy = 0
    s.vx = 0
    s.jump = 0 
    s.facing = 'e'
    s.score = 0
    s.z = 1
    s.state = 'live'
    
    return s

def do_friction(v,f):
    if v > 0: v -= f
    if v < 0: v += f
    if abs(v) < 0.5: v = 0
    return v


def elephant_loop(g,s):
    keys = pygame.key.get_pressed()
    
    if s.rect.bottom == s.ball.rect.top and s.ball.rect.left <= s.rect.centerx <= s.ball.rect.right:
        dx = s.rect.centerx - s.ball.rect.centerx
        #if dx < 0: s.ball.rect.centerx -= min(4,-dx)
        #elif dx > 0: s.ball.rect.centerx += min(4,dx)
        if dx < 0: s.rect.centerx += min(4,-dx)
        elif dx > 0: s.rect.centerx -= min(4,dx)

    
    #handle X
    dx = 0
    if keys[K_LEFT]: dx -= 1
    if keys[K_RIGHT]: dx += 1
    #facing
    if dx < 0:
        s.facing = 'w'
    if dx > 0:
        s.facing = 'e'
    #accel
    if s.rect.bottom == s.ball.rect.top and s.ball.rect.left <= s.rect.centerx <= s.ball.rect.right:
        s.vx += dx*2
    else:
        s.vx += dx*3
        #pass
    s.vx = max(-8,min(8,s.vx))
    #do it...
    s.rect.x += s.vx
    s.rect.centerx = max(0,min(SW,s.rect.centerx))
    if s.rect.bottom == s.ball.rect.top and s.ball.rect.left <= s.rect.centerx <= s.ball.rect.right:
    #if 1:
        s.ball.vx = s.vx
    #friction
    s.vx = do_friction(s.vx,1.0)
    
    base = s.ball.rect.bottom
    
    #handle Y
    jvy = -12
    if keys[K_UP] and s.jump < 12 and ((s.vy == 0 and s.rect.bottom == s.ball.rect.top) or (s.vy == jvy) or (s.rect.bottom == base)):
        if s.vy == 0 and s.jump == 0:
            g.game.sfx['elephant-jump'].play()
        s.jump += 1
        s.vy = jvy
    elif s.vy < 0:
        s.vy += 1
        s.jump = 0
    else:
        s.vy += 0.5
    s.rect.y += s.vy
    if (s.ball.rect.top+16) >= s.rect.bottom >= s.ball.rect.top and s.ball.rect.left <= s.rect.centerx <= s.ball.rect.right:
        s.rect.bottom = s.ball.rect.top
        s.vy = 0
    s.rect.bottom = min(base,s.rect.bottom)
    if s.rect.bottom == base:
        s.vy = 0
    
    if s.vy < 0:
        s.image = g.gfx['elephant-n%s'%s.facing]
    elif s.vy > 0:
        s.image = g.gfx['elephant-s%s'%s.facing]
    else:
        s.image = g.gfx['elephant-%s'%s.facing]
    
     
def ball_new(g,pos):
    #s = Sprite(g.gfx['ball0'],pygame.Rect(0,0,32,32))
    s = Sprite(g.gfx['ball96'],pygame.Rect(0,8,96,96-8))
    s.rect.centerx = pos[0]
    s.rect.top = pos[1]
    s.vx = 0
    s.loop = ball_loop
    s.a = 0
    s.z = 0
    
    g.sprites.append(s)
    return s

def ball_loop(g,s):
    s.rect.x += s.vx
    s.rect.centerx = max(0,min(SW,s.rect.centerx))
    s.vx = do_friction(s.vx,0.2)
    inc = -10
    #inc = 0
    if s.vx < 0:
        inc += 5
    if s.vx > 0:
        inc -= 5
    s.a = (s.a+inc+360) % 360
    s.image = g.gfx['ball-%d'%s.a]
    
def fruit_new(g,pos):
    #s = Sprite(g.gfx['fruit'],pygame.Rect(0,0,32,32))
    s = Sprite(g.gfx['babygiraffe'],pygame.Rect(0,0,20,73))
    s.rect.centerx = pos[0]
    s.rect.bottom = pos[1]
    s.vy = 2
    s.vx = rr(1,3)
    if s.rect.centerx > SW/2:
        s.vx = -s.vx
    s.loop = fruit_loop
    s.z = 2
    g.sprites.append(s)
    g.game.sfx['elephant-fall'].play()
    return s

def fruit_loop(g,s):
    s.rect.x += s.vx
    s.rect.y += s.vy
    s.vy += 0.1
    if s.rect.top > SH:
        g.sprites.remove(s)
        g.multi = 1
        return
    if s.rect.colliderect(g.player):
        g.sprites.remove(s)
        h = SH*2/3
        points = max(50,(h-g.player.rect.centery) * 1500 / h) * g.bonus * g.multi
        g.player.score += points
        g.game.sfx['elephant-giraffe'].play() 
        points_new(g,s.rect.center,points)
        #g.multi *= 1.15
        g.multi += 0.5
        #print g.player.score
        return
    
def points_new(g,pos,val):
    img = g.game.font2.render('+%d'%val,1,(0,0,0))
    s = Sprite(img,pygame.Rect(0,0,img.get_width(),img.get_height()))
    s.rect.centerx = pos[0]
    s.rect.centery = pos[1]
    s.frame = 0
    s.loop = points_loop
    s.z = 0
    g.sprites.append(s)
    return s

def points_loop(g,s):
    if s.frame == FPS:
        g.sprites.remove(s)
        
        return
    s.frame += 1
    
    
def spear_new(g,pos):
    #s = Sprite(g.gfx['spear'],pygame.Rect(0,0,32,32))
    s = Sprite(g.gfx['spear'],pygame.Rect(2,14,52,7))
    s.rect.centerx = pos[0]
    s.rect.centery = pos[1]
    s.vx = 6
    if s.rect.centerx > SW/2:
        s.vx = -s.vx
        s.image = g.gfx['spear-w']
    s.vy = -1
    s.loop = spear_loop
    s.z = 2
#    g.game.sfx['elephant-spear'].play()
    g.sprites.append(s)
    return s

def spear_loop(g,s):
    s.rect.x += s.vx
    s.rect.y += s.vy
    s.vy += 0.05
    if s.rect.left > SW:
        g.sprites.remove(s)
        return
    if s.rect.colliderect(g.player):
        if g.player in g.sprites:
            #NOTE: die
            g.game.sfx['elephant-pop'].play()
            #g.sprites.remove(g.player)
            g.player.state = 'dead'
    
class Level(engine.State):
    def __init__(self,game,parent):
        self.game = game
        self.parent = parent
        
    def init(self):
        self.bkgr = pygame.image.load(os.path.join('data','bg.png')).convert()
        self.gfx = {}
        for name in ['elephant','ball','fruit',
            'elephant-e','elephant-w',
            'elephant-ne','elephant-nw',
            'elephant-se','elephant-sw',
            'babygiraffe',
            'logo',
            'ball0','ball96','ball9690','ball96180','ball96270',
            'ball9645','ball96315','ball96225','ball96135',
            'spear','spear-w',
            ]:
            self.gfx[name] = pygame.image.load(os.path.join('data','%s.png'%name)).convert_alpha()
        self.gfx['ball960'] = self.gfx['ball96']
        
        magic = (255,0,255)
        #for a in xrange(0,360,45):
            #img = self.gfx['ball96%d'%a]
            #self.gfx['ball96%d'%a] = img2
            
        for a_ in xrange(0,360,5):
            inc = 45
            
            img1 = self.gfx['ball96%d'%((((359-a_)%360)/inc)*inc)]
            a = (a_)%inc
            i = pygame.transform.rotozoom(img1,a,1.0)
            r = pygame.Rect((i.get_width()-96)/2,(i.get_height()-96)/2,97,97)
            if r.right > i.get_width(): r.width -= 1
            if r.bottom > i.get_height(): r.height -= 1
            i1 = i.subsurface(r)
            
            #img2 = self.gfx['ball96%d'%((((359-(a_+inc)+360)%360)/inc)*inc)]
            #a = (a_-inc+360)%inc
            #i = pygame.transform.rotozoom(img2,a,1.0)
            #r = pygame.Rect((i.get_width()-96)/2,(i.get_height()-96)/2,97,97)
            #if r.right > i.get_width(): r.width -= 1
            #if r.bottom > i.get_height(): r.height -= 1
            #i2 = i.subsurface(r)
            
            #i3 = pygame.Surface((i2.get_width(),i2.get_height())).convert()
            #i3.fill(magic)
            #i3.set_colorkey(magic)
            #i3.blit(i2,(0,0))
            #i3.set_alpha(((a_%inc)*255/inc))
            
            #i1.blit(i3,(0,0))
            
            self.gfx['ball-%d'%a_] = i1
            
        #for a_ in xrange(0,360,5):
            #inc = 45
            #mod = -(inc/2)
            #i2 = img = self.gfx['ball96%d'%((((359-a_+mod+360)%360)/inc)*inc)]
            #a = (a_+mod+360)%inc
            #i = pygame.transform.rotozoom(img,a,1.0)
            #r = pygame.Rect((i.get_width()-96)/2,(i.get_height()-96)/2,97,97)
            #if r.right > i.get_width(): r.width -= 1
            #if r.bottom > i.get_height(): r.height -= 1
            #i2 = i.subsurface(r)
            #self.gfx['ball-%d'%a_] = i2
        
        self.view = pygame.Rect(0,0,SW,SH)
        self.sprites = []
        self.player = elephant_new(self,(SW/2,SH-120))
        self.frame = 0
        
        self.giraffe_speed = 2.0
        self.giraffe_next = FPS*1
        
        self.spear_speed = 7.0
        self.spear_next = FPS*3
        self.spear_n = 0
        self.bonus = 1
        self.multi = 1
        
        self.loop()
        self.paint(self.game.screen)
        import states
        return states.Pause(self.game,'get ready!',self)

    def loop(self):
        self.game.high = max(self.game.high,self.player.score)
        
        for s in self.sprites:
            if hasattr(s,'loop'):
                s.loop(self,s)
                
        inc = 20
                
        if self.frame == self.giraffe_next:
            fruit_new(self,(rr(0,SW),0))
            #fruit_new(self,(rr(SW*1/8,SW*7/8),0))
            self.giraffe_next += round(self.giraffe_speed * FPS)
        if self.frame > 1 and (self.frame%(FPS*inc)) == 0:
            self.giraffe_speed = max(0.5,self.giraffe_speed * 0.75)
            
        if self.frame == self.spear_next-30: # play sound ahead of spear throw
		    self.game.sfx['elephant-spear'].play()

        if self.frame == self.spear_next:
            y = self.player.rect.centery
            x = SW*(self.spear_n%2)
            spear_new(self,(x,rr(y-32,y+32)))
            self.spear_next += round(self.spear_speed * FPS)
            self.spear_n += 1
        if self.frame > 1 and (self.frame%(FPS*inc)) == 0:
            self.bonus += 1
            self.spear_speed = max(0.5,self.spear_speed * 0.8)
            
        if self.player.state == 'dead':
            #import states
            #return states.Prompt(self.game,'try again? y/n',Level(self.game,self.parent),self.parent)
            import states
            import highmenu
            self.game.score = self.player.score
            r = highmenu.Highs(self.game,False,'high scores',self.parent)
            return states.Pause(self.game,'game over',r)

        self.frame += 1
                
    def paint(self,screen):
        view = self.view
        sx = (self.frame*BLUR)%self.bkgr.get_width()
        screen.blit(self.bkgr,(-sx,0))
        screen.blit(self.bkgr,(self.bkgr.get_width()-sx,0))
        screen.blit(self.gfx['logo'],(0,0))
        sz = [[] for n in xrange(0,8)]
        for s in self.sprites[:]:
            sz[s.z].append(s)
        for ss in sz:
            for s in ss:
                x,y = s.rect.x-s.shape.x-view.x,s.rect.y-s.shape.y-view.y
                screen.blit(s.image,(x,y))
            
        fnt = self.game.font
        text = 'score: %06d'%self.player.score
        border = 2
        img = fnt.render(text,1,(0,0,0))
        x,y = (SW-img.get_width())/2,SH-img.get_height()-10 
        screen.blit(img,(x+border,y+border))
        img = fnt.render(text,1,(255,255,255))
        screen.blit(img,(x,y))
        pygame.display.flip()
        
    def update(self,screen):
        return self.paint(screen)
    
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_ESCAPE:
            import states
            return states.Prompt(self.game,'quit? y/n',self.parent,self)
    
 
