import os

import pygame
from pygame.locals import *

from cnst import *

from pgu import engine

import highmenu

def dotext(s,fnt,txt,c,pos):
    img = fnt.render(txt,1,(0,0,0))
    pos = (SW-img.get_width())/2,pos[1]
    s.blit(img,(pos[0]+2,pos[1]+2))
    img = fnt.render(txt,1,c)
    s.blit(img,pos)
    return pygame.Rect(pos[0],pos[1],img.get_width(),img.get_height())

def dotextc(s,fnt,txt,c):
    img = fnt.render(txt,1,(0,0,0))
    return dotext(s,fnt,txt,c,((SW-img.get_width())/2,(SH-img.get_height())/2))

class Menu(engine.State):
    def __init__(self,game):
        self.game = game
        
    def init(self):
        self.font = self.game.font11
        self.bkgr = pygame.image.load(os.path.join('data','bg.png')).convert()
        self.logo = pygame.image.load(os.path.join('data','logo.png')).convert_alpha()
        self.cur = 0
        self.lvl = 0
        self.items = [
            ('play','play'),
            ('high scores','high scores'),
            ('web highs','web highs'),
            ('web recent','web recent'),
            ('about','about'),
            ('quit','quit'),
            ]
        self.rects = []
        self.frame = 0
        
    def paint(self,screen):
        sx = (self.frame*BLUR)%self.bkgr.get_width()
        screen.blit(self.bkgr,(-sx,0))
        screen.blit(self.bkgr,(self.bkgr.get_width()-sx,0))
        
        screen.blit(self.logo,(0,0))
        
        x,y = 64,95
        score = 0
        if len(self.game.highs):
            score = self.game.highs[0].score

        line = 'high: %d'%score
        c = (255,255,255)
        r = dotext(screen,self.font,line,c,(x,y))
        y += 70
        
        n = 0
        self.rects = []
        for line,val in self.items:
            c = (0xaa,0xaa,0xaa)
            if n == self.cur:
                c = (0xff,0xcc,0x88)
            line = line.replace('$l',str(self.lvl+1))
            r = dotext(screen,self.font,line,c,(x,y))
            self.rects.append((r,n))
            y += 35
            n += 1
            
            
        c=(255,255,255)
        y = SH-42-18
        line = 'a "4 hour" #ludumdare production'
        dotext(screen,self.game.font2,line,c,(x,y))
        y += 21
        line = 'by philhassey, DrPetter, and trick'
        dotext(screen,self.game.font2,line,c,(x,y))
        
        pygame.display.flip()
        
    def update(self,screen):
        return self.paint(screen)
    def loop(self):
        self.game.score = 0
        self.frame += 1
        
    def event(self,e):
        mx = len(self.items)
        if e.type is KEYDOWN and e.key == K_UP:
            self.cur = (self.cur + mx - 1) % mx
            self.repaint()
            self.game.sfx['elephant-bounce'].play()
        if e.type is KEYDOWN and e.key == K_DOWN:
            self.cur = (self.cur + mx + 1) % mx
            self.repaint()
            self.game.sfx['elephant-bounce'].play()

        mx = 10
        if e.type is KEYDOWN and e.key == K_LEFT:
            self.lvl = (self.lvl + mx - 1) % mx
            self.repaint()
        if e.type is KEYDOWN and e.key == K_RIGHT:
            self.lvl = (self.lvl + mx + 1) % mx
            self.repaint()
            
        if e.type is MOUSEMOTION:
            for r,n in self.rects:
                if r.collidepoint(e.pos):
                    self.cur = n
                    self.repaint()
                    
        if e.type is MOUSEBUTTONDOWN or (e.type is KEYDOWN and e.key == K_RETURN):
            self.game.sfx['elephant-mselect'].play()
            cmd = self.items[self.cur][1]
            if cmd == 'play':
                import level
                return level.Level(self.game,self)
            if cmd == 'quit':
                return engine.Quit(self.game)
            val = cmd
            
            if val == 'high scores':
                return highmenu.Highs(self.game,False,val,self)
            
            if val == 'web highs':
                return highmenu.Highs(self.game,'list',val,self) #global
            
            if val == 'web recent':
                return highmenu.Highs(self.game,'recent',val,self) #recent
            
            if val == 'about':
                return About(self.game,self)
            


class About(engine.State):
    def __init__(self,game,parent):
        self.game = game
        self.parent = parent
    def init(self):
        self.bkgr = self.game.bkgr
    def paint(self,screen):
        screen.fill((255,255,255))
        img = self.bkgr
        img.set_alpha(128)
        screen.blit(img,(0,0))
        
        fg = (0xff,0x55,0x55)
        bg = (0,0,0)
        fnt = self.game.font
        text = 'about'
        img = fnt.render(text,1,fg)
        img2 = fnt.render(text,1,bg)
        y = 12
        x = (SW-img.get_width())/2
        screen.blit(img2,(x+2,y+2))
        screen.blit(img,(x,y))

        fnt = self.game.font2
        y = 80
        c = (0,0,0)
        help = [
            'a "4 hour" #ludumdare production',
            '',
            'philhassey - code',
            'DrPetter - music, sfx',
            'trick - gfx',
            '',
            'save the giraffes! ',
            'watch out for the angry natives! ',
            'use the arrows to move and jump. ',
            'escape to pause / quit',
            '',
            'www.imitationpickles.org/elephant/',
            ]

        for line in help:
            img = fnt.render(line,1,c)
            #img2 = fnt.render(line,1,bg)
            x = (SW-img.get_width())/2
            #screen.blit(img2,(x+2,y+2))
            screen.blit(img,(x,y))
            y += 26
        pygame.display.flip()
    def event(self,e):
        if (e.type is KEYDOWN and e.key in (K_RETURN,K_ESCAPE)) or e.type is MOUSEBUTTONDOWN:
            return self.parent


    
class Prompt(engine.State):
    def __init__(self,game,text,yes,no):
        self.game = game
        self.text = text
        self.yes = yes
        self.no = no
        
    def paint(self,screen):
        dotextc(screen,self.game.font,self.text,(255,255,255))
        pygame.display.flip()
        
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_y:
            return self.yes
        if e.type is KEYDOWN and e.key == K_n:
            return self.no
        
class Pause(engine.State):
    def __init__(self,game,text,next):
        self.game = game
        self.text = text
        self.next = next
        
    def paint(self,screen):
        dotextc(screen,self.game.font,self.text,(255,255,255))
        pygame.display.flip()
        
    def event(self,e):
        if e.type is KEYDOWN or e.type is MOUSEBUTTONDOWN:
            return self.next
